export enum UserRole {
	ADMIN = 2,
	USER = 1,
	DISABLED = 0
}

export interface UploadFile {
	parent: number;
	fullName: string;
	file: File;
}

// eslint-disable-next-line @typescript-eslint/no-namespace
export namespace Requests {
	// eslint-disable-next-line @typescript-eslint/no-empty-interface
	export interface Base {}

	export interface Admin extends Base {
		user: number;
	}

	export interface SetUserRole extends Admin {
		role: UserRole;
	}

	export interface SignUp extends Base {
		username: string;
		password: string;
	}

	export interface Login extends SignUp {
		otp?: string;
	}

	export interface TfaSetup extends Base {
		mail: boolean;
	}

	export interface TfaComplete extends Base {
		mail: boolean;
		code: string;
	}

	export interface ChangePassword extends Base {
		oldPassword: string;
		newPassword: string;
	}

	export interface CreateFolder extends Base {
		parent: number;
		name: string;
	}

	export interface CreateZip extends Base {
		nodes: number[];
	}
}

// eslint-disable-next-line @typescript-eslint/no-namespace
export namespace Responses {
	export interface Base {
		statusCode: number;
	}

	export interface Success extends Base {
		statusCode: 200;
	}

	export interface Error extends Base {
		statusCode: 400 | 401 | 403;
		message?: string;
	}

	export interface Login extends Success {
		jwt: string;
	}

	export interface RequestsTotpTfa extends Success {
		qrCode: string;
		secret: string;
	}

	export interface GetRoot extends Success {
		rootId: number;
	}

	export interface GetNodeEntry {
		id: number;
		name: string;
		isFile: boolean;
		preview: boolean;
		parent: number | null;
		size?: number;
	}

	export interface GetNode extends Success, GetNodeEntry {
		children?: GetNodeEntry[];
	}

	export interface PathSegment {
		path: string;
		node?: number;
	}

	export interface GetPath extends Success {
		segments: PathSegment[];
	}

	export interface CreateFolder extends Success {
		id: number;
	}

	export interface CreateFolderExists extends Success {
		exists: true;
		id: number;
		isFile: boolean;
	}

	export interface CreateZip extends Success {
		done: boolean;
		progress?: number;
		total?: number;
	}

	export interface DownloadBase64 extends Success {
		data: string;
	}

	export interface GetType extends Success {
		type: string;
	}

	export interface UserInfo extends Success {
		name: string;
		gitlab: boolean;
		tfaEnabled: boolean;
	}

	export interface GetUsersEntry {
		id: number;
		gitlab: boolean;
		name: string;
		role: UserRole;
		tfaEnabled: boolean;
	}

	export interface GetUsers extends Success {
		users: GetUsersEntry[];
	}
}
